#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'rubygems'
require 'active_record'

ActiveRecord::Base.establish_connection(
  :adapter => 'mysql',
  :host => '127.0.0.1',
  :database => 'webshop'
)



class Bouquet < ActiveRecord::Base
  has_and_belongs_to_many :flowers

  def price
    flowers.inject(self.base_price) do |total, f| 
      total += f.price * f.quantity.to_i
    end
  end

  def add(flower, quantity)
    self.flowers.push_with_attributes(
      flower,
      :quantity => quantity
    )
  end
end

class Flower < ActiveRecord::Base
  has_and_belongs_to_many :bouquets
  belongs_to :stock_item
end

class StockItem < ActiveRecord::Base
  has_one :flower
end



class Stock
  def add_flower(name, price, quantity)
    si = StockItem.new(:quantity => quantity)
    si.save
    si.create_flower(
      :name => name,
      :price => price
    )
  end
  def remove(bouquet)
    Bouquet.transaction do 
      bouquet.flowers.each do |f|
        si = StockItem.find(f.stock_item_id)
        if si.quantity < f.quantity.to_i
          raise 'Not enough flowers left!'
        end
        si.quantity -= f.quantity.to_i
        si.save
      end
    end
  end

  def print_report
    StockItem.find(:all).each do |si| 
      puts "#{si.flower.name}: #{si.quantity} in stock."
    end
  end
end


Flower.find(:all).each { |f| f.destroy }
StockItem.find(:all).each { |si| si.destroy }


stock = Stock.new
[
  ['rose', 1.10, 1000],
  ['violet', 0.40, 2000],
  ['sunflower', 0.40, 6]
].each do |name, price, quantity|
  stock.add_flower(name, price, quantity)
end
stock.print_report



bouquet = Bouquet.new(
  :name => 'Honeybunch',
  :base_price => 1.37
)
bouquet.save
bouquet.add(Flower.find_by_name('rose'), 1)
bouquet.add(Flower.find_by_name('sunflower'), 6)
puts "'#{bouquet.name}' costs $#{bouquet.price}."



stock.remove(bouquet)
stock.print_report


# vim:sw=2:tw=0

