-- START:customers
create table customers(
   id int unsigned not null auto_increment primary key,
   name varchar(64) not null,
   email varchar(64) not null,
   created timestamp not null
);
-- END:customers

-- START:orders
create table orders(
   id int unsigned not null auto_increment primary key,
   customer_id int not null,
   created timestamp not null
);
alter table orders add constraint fk1_customers
  foreign key (customer_id) references customers(id);
-- END:orders

-- START:delivery_addresses
create table delivery_addresses(
   id int unsigned not null auto_increment primary key,
   order_id int not null,
   name varchar(64) not null,
   street varchar(64) not null,
   postal_code varchar(16) not null,
   city varchar(64) not null,
   state varchar(64),
   country varchar(64)
);
alter table delivery_addresses add constraint fk1_orders
  foreign key (order_id) references orders(id);
-- END:delivery_addresses

-- START:order_items
create table order_items(
   id int unsigned not null auto_increment primary key,
   order_id int not null,
   bouquet_id int not null,
   amount int not null default 1
);
alter table order_items add constraint fk1_orders
  foreign key (order_id) references orders(id);
alter table order_items add constraint fk1_bouquets
  foreign key (bouquet_id) references bouquets(id);
-- END:order_items

-- START:stock_items
create table stock_items(
   id int unsigned not null auto_increment primary key,
   quantity int not null
);
-- END:stock_items

-- START:bouquets
create table bouquets(
   id int unsigned not null auto_increment primary key,
   name varchar(64) not null,
   base_price double not null
);
-- END:bouquets

-- START:bouquets_flowers
create table bouquets_flowers(
   bouquet_id int not null,
   flower_id int not null
);
alter table bouquets_flowers add constraint fk1_bouquet
  foreign key (bouquet_id) references bouquets(id);
alter table bouquets_flowers add constraint fk2_flower
  foreign key (flower_id) references flowers(id);
-- END:bouquets_flowers

-- START:flowers
create table flowers(
   id int unsigned not null auto_increment primary key,
   name varchar(64) not null,
   price double not null
);
-- END:flowers

