#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'dbi'
require 'singleton'

class Database
  include Singleton
  attr_reader :connection

  def initialize() @connection = nil; end

  def connect(usr, pwd, db)
    @connection = DBI.connect("DBI:Mysql:#{db}", usr, pwd)
  end

  def disconnect
    @connection.disconnect if !@connection.nil?
    @connection = nil
  end
end



class Flower
  attr_reader :id
  attr_accessor :name, :price
  
  def Flower.create(name, price)
    connection = Database.instance.connection
    @@create_flower ||= connection.prepare(<<-SQL) 
    insert into flowers (name, price) values(?,?)
    SQL
    @@create_flower.execute(name, price) 
    flower_id = connection.func(:insert_id) 
    Flower.new(flower_id, name, price)
  end

  def to_s
    "A #{@name} (#{@id}) costs $#{@price}."
  end
  
  private

  def initialize(id, name, price)
    @id, @name, @price = id, name, price
  end
end



class Flower
  def Flower.find(name)
    connection = Database.instance.connection    
    @@find_flower ||= connection.prepare(<<-SQL)
    select id, name, price from flowers where name = ?
    SQL
    @@find_flower.execute(name)
    flower = @@find_flower.fetch
    return nil if flower.nil?
    Flower.new(*flower)
  end

  def update
    connection = Database.instance.connection
    @@update_flower ||= connection.prepare(<<-SQL)
    update flowers set name = ?, price = ? where id = ?
    SQL
    @@update_flower.execute(@name, @price, @id)
  end

  def destroy
    connection = Database.instance.connection
    @@destroy_flower ||= connection.prepare(<<-SQL)
    delete from flowers where id = ?
    SQL
    @@destroy_flower.execute(@id)
  end
end


if $0 == __FILE__
  
  Database.instance.connect('', '', 'webshop')
  rose = Flower.create('rose', 1.99)
  puts rose
  

  
  rose = Flower.find('rose')
  rose.price = 2.49
  rose.update

  rose = Flower.find('rose')
  puts rose
  rose.destroy
  puts Flower.find('rose')
  Database.instance.disconnect
  
end
# vim:sw=2:tw=0

