#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'rubygems'
require 'active_record'

ActiveRecord::Base.establish_connection(
  :adapter => 'mysql',
  :host => '127.0.0.1',
  :database => 'webshop'
)



class Bouquet < ActiveRecord::Base
  has_and_belongs_to_many :flowers
end

class Flower < ActiveRecord::Base
  has_and_belongs_to_many :bouquets
end


Flower.find(:all).each { |f| f.destroy }

[
  ['rose', 1.10],
  ['violet', 0.40],
  ['sunflower', 0.40],
  ['clove', 0.65],
  ['lily', 0.80]
].each do |name, price|
  flower = Flower.new(:name => name, :price => price)
  flower.save
end


rose = Flower.find_by_name('rose')
sunflower = Flower.find_by_name('sunflower')
bouquet = Bouquet.new
bouquet.name = 'Honeybunch'
bouquet.base_price = 1.37;
bouquet.save
bouquet.flowers.push_with_attributes(rose, :quantity => 1)
bouquet.flowers.push_with_attributes(sunflower, :quantity => 6)



b = Bouquet.find_by_name('Honeybunch')
puts "'#{b.name}' contains #{b.flowers.size} different flowers:"
b.flowers.each { |f| puts "#{f.name} (#{f.quantity})" }



class Bouquet
  def price
    flowers.inject(self.base_price) { |total, f| 
      total += f.price * f.quantity.to_i
    }
  end
end
b = Bouquet.find_by_name('Honeybunch')
puts "'#{b.name}' costs $#{b.price}."


b.destroy
exit
# vim:sw=2:tw=0

