#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'rubygems'
require 'active_record'

ActiveRecord::Base.establish_connection(
  :adapter => 'mysql',
  :host => '127.0.0.1',
  :database => 'webshop'
)



class Bouquet < ActiveRecord::Base
  has_and_belongs_to_many :flowers
end
class Flower < ActiveRecord::Base
  has_and_belongs_to_many :bouquets
  belongs_to :stock_item 
end
class StockItem < ActiveRecord::Base
  has_one :flower 
end



class StockItem < ActiveRecord::Base
  def to_s
    "#{self.id}: #{self.quantity} in stock."
  end
end


Flower.find(:all).each { |f| f.destroy }
StockItem.find(:all).each { |si| si.destroy }


[
  ['rose', 1.10, 1000],
  ['violet', 0.40, 2000],
  ['sunflower', 0.40, 500],
  ['clove', 0.65, 2000],
  ['lily', 0.80, 1500]
].each do |name, price, quantity|
  si = StockItem.new(:quantity => quantity)
  si.save
  Flower.new(
    :stock_item => si, 
    :name => name,
    :price => price
  ).save
end



puts "Stock items:"
StockItem.find(:all).each { |si|
  puts "#{si.flower.name}: #{si.quantity} in stock." 
}



rose = Flower.find_by_name('rose')
sunflower = Flower.find_by_name('sunflower')
bouquet = Bouquet.new
bouquet.name = 'Honeybunch'
bouquet.base_price = 1.37;
bouquet.save
bouquet.flowers.push_with_attributes(rose, :quantity => 1)
bouquet.flowers.push_with_attributes(sunflower, :quantity => 6)



b = Bouquet.find_by_name('Honeybunch')
puts "'#{b.name}' contains #{b.flowers.size} different flowers:"
b.flowers.each { |f| puts "#{f.name} (#{f.quantity})" }



class Bouquet
  def price
    flowers.inject(self.base_price) { |total, f| 
      total += f.price * f.quantity.to_i
    }
  end
end
b = Bouquet.find_by_name('Honeybunch')
puts "'#{b.name}' costs $#{b.price}."


def remove(bouquet)
  bouquet.flowers.each do |f|
    si = StockItem.find(f.stock_item_id)
    si.quantity -= f.quantity.to_i
    si.save
  end
end

remove(b)
puts "Stock items:"
StockItem.find(:all).each { |si| 
  puts si
  puts si.flower
}

b.destroy
exit
# vim:sw=2:tw=0

