#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'rubygems'
require 'active_record'

ActiveRecord::Base.establish_connection(
  :adapter => 'mysql',
  :host => '127.0.0.1',
  :database => 'webshop'
)



class Bouquet < ActiveRecord::Base
  has_and_belongs_to_many :flowers

  def validate
    errors.add_on_empty %w(name base_price)
    unless base_price >= 0.0
      errors.add('base_price', 'is negative')
    end
    errors.add('flowers', 'must exist') unless flowers.size > 0
  end
end

class Flower < ActiveRecord::Base
  has_and_belongs_to_many :bouquets

  def validate
    errors.add_on_empty %w(name price)
    errors.add(:price, 'is negative') unless price >= 0.0
  end
end


Flower.find(:all).each { |f| f.destroy }

[
  ['rose', 1.10],
  ['violet', 0.40],
  ['sunflower', 0.30],
  ['clove', 0.65],
  ['lily', 0.80]
].each do |name, price|
  flower = Flower.new(:name => name, :price => price)
  flower.save
end


print 'Trying to save nameless flower: '
puts Flower.new(:name => nil, :price => 0).save
print 'Trying to save really cheap flower: '
puts Flower.new(:name => 'cheap', :price => -1).save
print 'Trying to save a valid flower: '
puts Flower.new(:name => 'daisy', :price => 0.02).save



rose = Flower.find_by_name('rose')
sunflower = Flower.find_by_name('sunflower')
bouquet = Bouquet.new
bouquet.name = 'Honeybunch'
bouquet.base_price = 1.37
print 'Trying to save bouquet without flowers: '
puts bouquet.save
bouquet.flowers.push_with_attributes(rose, :quantity => 1)
bouquet.flowers.push_with_attributes(sunflower, :quantity => 6)
print 'Trying to save valid bouquet: '
puts bouquet.save


bouquet.destroy
exit
# vim:sw=2:tw=0

