#!/usr/local/bin/ruby -w
#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---


require 'dbi'

Customer = Struct.new(:id, :name, :surname, :email)



class Customer
  def to_csv(del = ';')
    "#{name}#{del}#{surname}#{del}#{email}"
  end
end



class CustomerFinder
  def initialize(connection)
    @find_stmt = connection.prepare(<<-SQL)
# STOP_SYNTAX

    select a.id, a.name, a.surname, a.email
    from   customers a, orders b
    where  a.id = b.customer_id
    and    b.created < sysdate - :days
    and    b.created = (
      select max(created)
      from   orders
      where  customer_id = a.id
    )
    SQL
# START_SYNTAX
  end


  def find(days) 
    @find_stmt.bind_param(':days', days)
    @find_stmt.execute
    customers = []
    while row = @find_stmt.fetch do
      customers << Customer.new(*row)
    end
    customers
  end
end


if $0 == __FILE__
  
  connection = DBI.connect('DBI:Oracle', 'maik', 'maik')
  finder = CustomerFinder.new(connection)
  customers = finder.find(180)
  customers.each { |c| puts c.email }
  connection.disconnect
  
end

# vim:sw=2:tw=0
