#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'rubygems'
require 'activeldap'

ActiveLDAP::Base.connect(
  :base => 'dc=pragbouquet,dc=com',
  :bind_format => 'cn=root,dc=pragbouquet,dc=com',
  :password_block => Proc.new { 'secret' },
  :allow_anonymous => false
)



class Group < ActiveLDAP::Base 
  ldap_mapping :dnattr => 'cn', :prefix => 'ou=Groups' 
end

customers = Group.new('customers') 
puts "The 'customers' group has the following attributes:"
customers.attributes.each { |a| puts "  #{a}" } 

puts "\nIts group id is #{customers.gidNumber}."

puts "\nWe have the following groups:"
Group.find_all('*').each { |g| puts "  #{g}" } 


puts '-' * 80


class Customer < ActiveLDAP::Base
  ldap_mapping :dnattr => 'uid',
               :prefix => 'cn=customers,ou=Groups'
end

puts "Our customers are:"
Customer.find_all('*').each { |c| puts "  #{c}" }
h = Customer.new('homer@example.com')
puts "\nCommon name attribute:"
p h.cn 
p h.cn(true) 



def get_unused_accounts(used_accounts)
  unused_accounts = []
  Customer.find_all('*').each do |email|
    unused_accounts << email if !used_accounts.include?(email)
  end
  unused_accounts
end

puts get_unused_accounts(['homer@example.com'])



def delete_unused_accounts(used_accounts)
  Customer.find_all('*').each do |cn|
    Customer.new(cn).delete if !used_accounts.include?(cn)
  end
end

#puts delete_unused_accounts(['homer@example.com'])


def mark_unused_accounts(used_accounts)

  Customer.find_all('*').each do |cn|
    if !used_accounts.include?(cn)
      c = Customer.new(cn)
      c.description = 'unused'
      c.write
    end
  end
end

puts mark_unused_accounts(['homer@example.com'])
