-- START:create_example
create table countries(
  country_code varchar(2) primary key,
  name varchar(128) not null
);

create table customers(
  id int primary key,
  name varchar(64) not null,
  surname varchar(128) not null,
  street varchar(128) not null,
  house_number varchar(10) not null,
  postal_code varchar(10) not null,
  city varchar(128) not null,
  state varchar(20),
  country_code varchar(2) not null,
  email varchar(128) not null,
  created date default sysdate
);
alter table customers add constraint fk_customer
  foreign key (country_code) references countries(country_code);
-- END:create_example

create sequence customer_id;

-- START:fill_countries
insert into countries values('US', 'United States');
insert into countries values('DE', 'Germany');
insert into countries values('IT', 'Italy');
insert into countries values('FR', 'France');
insert into countries values('GB', 'United Kingdom');
-- END:fill_countries

-- START:fill_customers
insert into customers values(
  1,
  'Homer',
  'Simpson',
  'Evergreen Terrace',
  '742',
  '1111',
  'Springfield',
  'xx',
  'US',
  'homer@example.com',
  sysdate
);
-- END:fill_customers

insert into customers values(
  2,
  'Barney',
  'Gumble',
  'Booze Place',
  '2',
  '1111',
  'Springfield',
  'xx',
  'US',
  'barney@example.com',
  sysdate
);

insert into customers values(
  3,
  'Nad',
  'Flanders',
  'Evergreen Terrace',
  '740',
  '1111',
  'Springfield',
  'xx',
  'US',
  'nad@example.com',
  sysdate
);

insert into customers values(
  4,
  'Selma',
  'Bouvier',
  'Short Road',
  '456',
  '1111',
  'Springfield',
  'xx',
  'US',
  'selma@example.com',
  sysdate
);

insert into customers values(
  5,
  'Seymour',
  'Skinner',
  'Elm Street',
  '23',
  '1111',
  'Springfield',
  'xx',
  'US',
  'seymour@example.com',
  sysdate
);

insert into customers values(
  6,
  'Julius',
  'Hibbert',
  'Main Street',
  '15',
  '1111',
  'Springfield',
  'xx',
  'US',
  'julius@example.com',
  sysdate
);

create table login_data(
  customer_id int primary key,
  user_name varchar(64) not null,
  password varchar(128) not null
);
alter table login_data add constraint fk_login_data
  foreign key (customer_id) references customers(id);

create table orders(
  id int primary key,
  customer_id int not null,
  state int not null,
  created timestamp default sysdate
);
alter table orders add constraint fk_order
  foreign key (customer_id) references customers(id);

create sequence order_id;

insert into orders values(order_id.nextval, 1, 0, to_date('2004-02-10', 'YYYY-MM-DD'));
insert into orders values(order_id.nextval, 2, 0, to_date('2005-03-23', 'YYYY-MM-DD'));
insert into orders values(order_id.nextval, 3, 0, to_date('2004-12-20', 'YYYY-MM-DD'));
insert into orders values(order_id.nextval, 3, 0, to_date('2005-02-10', 'YYYY-MM-DD'));
insert into orders values(order_id.nextval, 3, 0, to_date('2005-05-22', 'YYYY-MM-DD'));
insert into orders values(order_id.nextval, 4, 0, to_date('2005-06-01', 'YYYY-MM-DD'));
insert into orders values(order_id.nextval, 5, 0, to_date('2004-01-02', 'YYYY-MM-DD'));
insert into orders values(order_id.nextval, 5, 0, to_date('2004-09-30', 'YYYY-MM-DD'));
insert into orders values(order_id.nextval, 6, 0, to_date('2005-03-06', 'YYYY-MM-DD'));

create table order_items(
  id int primary key,
  order_id int not null,
  item varchar2(128) not null,
  amount int not null  
);
alter table order_items add constraint fk_order_item
  foreign key (order_id) references orders(id);

create sequence order_item_id;

disconnect;
exit;

