#!/usr/local/bin/ruby -w
#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'database'


Customer = Struct.new(:id, :name, :surname, :street, :house_number,
  :postal_code, :city, :state, :country_code, :email, :created)


class Customer
  def to_s
    "#{id}: #{name}, #{surname} (#{email})"
  end
end


class Customer
  def Customer.read(customer_id)
    find_sql = <<-ENDSQL
    select id, name, surname, street, house_number, postal_code,
      city, state, country_code, email, created
    from customers where id = #{customer_id}
    ENDSQL

    connection = Database.instance.connection 
    cursor = connection.exec(find_sql)
    row = cursor.fetch
    cursor.close
    raise "Customer #{customer_id} not found!" if row.nil?
    Customer.new(*row)
  end
end



class Customer
  PREP_FIND_SQL = <<-ENDSQL
  select id, name, surname, street, house_number, postal_code,
         city, state, country_code, email, created
  from customers where id = :id
  ENDSQL
  
  def Customer.find(customer_id)
    connection = Database.instance.connection 
    @find_sql ||= connection.parse(PREP_FIND_SQL)
    @find_sql.bind_param(':id', customer_id)
    @find_sql.exec
    row = @find_sql.fetch
    raise "Customer #{customer_id} not found!" if row.nil?
    Customer.new(*row)
  end
end



class Customer
  UPDATE_SQL = <<-ENDSQL
  update customers
  set name = :1, surname = :2, street = :3, house_number = :4,
    postal_code = :5, city = :6, state = :7, country_code = :8,
    email = :9
  where id = :10
  ENDSQL
  
  def update
    @update_sql ||= Database.instance.connection.parse(UPDATE_SQL)
    @update_sql.bind_param(1, self.name)
    @update_sql.bind_param(2, self.surname)
    @update_sql.bind_param(3, self.street)
    @update_sql.bind_param(4, self.house_number)
    @update_sql.bind_param(5, self.postal_code)
    @update_sql.bind_param(6, self.city)
    @update_sql.bind_param(7, self.state)
    @update_sql.bind_param(8, self.country_code)
    @update_sql.bind_param(9, self.email)
    @update_sql.bind_param(10, self.id)
    @update_sql.exec
  end
end



class Customer
  INSERT_SQL = <<-ENDSQL
  insert into customers values(
    customer_id.nextval, :name, :surname, :street, :house_number,
    :postal_code, :city, :state, :country_code, :email, sysdate
  ) returning id into :new_id
  ENDSQL

  def save
    @insert_sql ||= Database.instance.connection.parse(INSERT_SQL)
    @insert_sql.bind_param(1, self.name)
    @insert_sql.bind_param(2, self.surname)
    @insert_sql.bind_param(3, self.street)
    @insert_sql.bind_param(4, self.house_number)
    @insert_sql.bind_param(5, self.postal_code)
    @insert_sql.bind_param(6, self.city)
    @insert_sql.bind_param(7, self.state)
    @insert_sql.bind_param(8, self.country_code)
    @insert_sql.bind_param(9, self.email)
    @insert_sql.bind_param(':new_id', nil, Fixnum)
    @insert_sql.exec
    self.id = @insert_sql[':new_id']
  end
end



class Customer
  DELETE_SQL = 'delete from customers where id = :id'

  def delete
    @delete_sql ||= Database.instance.connection.parse(DELETE_SQL)
    @delete_sql.exec(self.id)
  end
end


if $0 == __FILE__
  Database.instance.connect('maik', 'maik')
  Database.instance.connection.exec('delete from customers')
  c = Customer.new
  c.name = 'Homer'
  c.surname = 'Simpson'
  c.street = 'Evergreen Terace'
  c.house_number = '42'
  c.postal_code = '1111'
  c.city = 'Springfield'
  c.state = 'xx'
  c.country_code = 'US'
  c.email = 'homer@example.com'
  current_id = c.save
  puts c

  c.name = 'Marge'
  c.update
  c = Customer.find(current_id)
  puts c

  c.delete
  c = Customer.find(current_id)
  Database.instance.disconnect
end

# vim:sw=2:tw=0
