#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---
require 'ldap'
require 'ldap/ldif'
include LDAP

User = Struct.new(:uid, :forename, :surname)

Recipient = Struct.new(
  :forename, :surname, :street, :postal_code,
  :city, :state, :description
)

class AddressBook
  include Enumerable

  BASE_DC = 'dc=pragbouquet,dc=com'

  attr_reader :user
  
  def initialize(connection, user)
    @connection, @user = connection, user
  end
  
  def AddressBook.create(connection, user)
    cn = user.forename + ' ' + user.surname
    adr_book = []    
    [
      ['objectclass', %w(top person uidObject)],
      ['uid', [user.uid]],
      ['cn', [cn]],
      ['sn', [user.surname]],
      ['description', ['Address book of ' + cn]]
    ].each do |attr, values|
      adr_book << LDAP.mod(LDAP_MOD_ADD, attr, values)
    end

    connection.add(
      'uid=' + user.uid + ',' + BASE_DC,
      adr_book
    )
    AddressBook.new(connection, user)
  end

  def udn
    'uid=' + @user.uid + ',' + BASE_DC
  end
  
  def add(recipient)
    cn = recipient.forename + ' ' + recipient.surname
    entry = {
      'objectclass' => %w(top residentialPerson),
      'cn' => [cn],
      'sn' => [recipient.surname],
      'l' => [recipient.city],
      'street' => [recipient.street],
      'postalCode' => [recipient.postal_code],
      'st' => [recipient.state || ''],
      'description' => [recipient.description || '']
    }
    puts "#{cn}"
    p entry
    puts udn
    @connection.add('cn=' + cn + ',' + udn(), entry)
  end

  def each
    @connection.search(
      udn,
      LDAP_SCOPE_ONELEVEL,
      '(objectClass=residentialPerson)',
      nil, false, 0, 0,
      'sn'
    ) do |recipient|
      yield recipient
    end
  end
  
  def each_recipient
    each do |entry|
      rec_data = entry.to_hash
      sn = rec_data['sn'][0]
      cn = rec_data['cn'][0]
      cn.sub!(Regexp.new(' ' + sn + '$'), '')
      yield Recipient.new(
        cn,
        sn,
        rec_data['street'][0],
        rec_data['postalCode'][0],
        rec_data['l'][0],
        rec_data['st'][0],
        rec_data['description'][0]
      )
    end
  end

  def to_ldif
    inject('') { |ldif,e| ldif << e.to_ldif }
  end

  def delete() @connection.delete(udn); end

  def remove(recipient)
    cn = recipient.forename + ' ' + recipient.surname
    @connection.delete('cn=' + cn + ',' + udn())
  end

  def modify(recipient)
    cn = recipient.forename + ' ' + recipient.surname
    entry = {
      'l' => [recipient.city],
      'street' => [recipient.street],
      'postalCode' => [recipient.postal_code],
      'st' => [recipient.state],
      'description' => [recipient.description]
    }
    @connection.modify('cn=' + cn + ',' + udn(), entry)
  end
end
