/***
 * Excerpted from "Enterprise Integration with Ruby"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
***/
import java.io.*;
import java.net.*;

public class StatusMonitor {
    public static final String WARN = "w";
    public static final String ERROR = "e";
    public static final String FATAL = "f";

    public StatusMonitor(final String host, final int port)
        throws UnknownHostException, IOException
    {
        _sm = new Socket(host, port);
        _out = new PrintWriter(_sm.getOutputStream(), true);
        _in = new BufferedReader(
            new InputStreamReader(_sm.getInputStream())
        );
    }

    public int warn(final String app, final String msg)
        throws IOException
    {
        return log(WARN, app, msg);
    }
    
    public int error(final String app, final String msg)
        throws IOException
    {
        return log(ERROR, app, msg);
    }
    
    public int fatal(final String app, final String msg)
        throws IOException
    {
        return log(FATAL, app, msg);
    }
    
    public void terminate() throws IOException {
        _out.close();
        _in.close();
        _sm.close();
    }
    
    private int log(
        final String level, 
        final String app, 
        final String msg) throws IOException
    {
        final String DEL = ",";
        _out.println(level + DEL + app + DEL + msg);
        final String response = _in.readLine();
        int return_code = 1;
        try {
            return_code = Integer.parseInt(response);
        }
        catch(NumberFormatException ignoreMe) {}
        return return_code;
    }
        
    private Socket _sm;
    private PrintWriter _out;
    private BufferedReader _in;
}
