#!/usr/bin/ruby -w
#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---


require 'rubygems'
require 'active_record'
require 'gserver'
require 'tmail'
require 'net/smtp'


ActiveRecord::Base.establish_connection(
  :adapter => 'mysql',
  :host => '127.0.0.1',
  :database => 'smon'
)

class LogEntry < ActiveRecord::Base; end

class StatusMonitor < GServer
   def initialize(host = '127.0.0.1', port = '3333')
      super(port, host)
      @level_map = { 
         'w' => 'warn', 'e' => 'error', 'f' => 'fatal'
      }
   end
   
   # Expects CSV data in the following format:
   # level,application,message.
   def serve(client)
      level, app, msg = client.readline.chomp.split(',', 3)
      if @level_map.has_key?(level)
         entry = LogEntry.new
         entry.application = app
         entry.level = @level_map[level]
         entry.message = msg
         entry.save!

         inform_helpdesk(app, level, msg) if level == 'f'
         client.puts('0')
      else
         client.puts('1')
      end
   end
   
   private

   def inform_helpdesk(app, level, msg)
      subject = "A fatal error occurred in #{app}!"
      subject << " Regret your sins!"
      mail = create_mail(
         'helpdesk@pragbouquet.com',
         subject,
         msg
      )
      send_mail('localhost', mail)
   end
   
   def create_mail(to, subject, body)
     mail = TMail::Mail.new
     mail.date = Time.now
     mail.mime_version = '1.0'
     mail.set_content_type 'text', 'plain'
     mail.from = 'status_monitor@pragbouquet.com'
     mail.to = to
     mail.subject = subject
     mail.body = body
     mail
   end

   def send_mail(host, mail)
     msg = mail.encoded 
     Net::SMTP.start(host, 25) do |smtp| 
       smtp.send_mail(msg, mail.from_address, mail.destinations)
     end 
   end
end


sm = StatusMonitor.new
sm.start
sm.join
