#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'authenticator'
require 'authheader'

class ServerAuthHeaderHandler < AuthHeaderHandler
  @authenticator = Authenticator.new 

  def self.create 
    new(@authenticator)
  end

  def initialize(authenticator)
    super()
    @authenticator = authenticator
    @userid = @sessionid = nil
    @mustunderstand = true 
  end

  def on_simple_inbound(auth_header, mustunderstand)
    authenticated = false
    userid = auth_header['userid']
    passwd = auth_header['passwd']
    if @authenticator.login(userid, passwd)
      authenticated = true
    elsif sessionid = auth_header['sessionid']
      if userid = @authenticator.authenticate(sessionid)
        @authenticator.destroy_session(sessionid)
        authenticated = true
      end
    end
    unless authenticated
      raise RuntimeError.new('Authentication failed!')
    end
    @userid = userid
    @sessionid = @authenticator.create_session(userid)
  end

  def on_simple_outbound
    { 'sessionid' => @sessionid }
  end
end



require 'soap/rpc/standaloneServer'
require 'stock'

class StockServer < SOAP::RPC::StandaloneServer
  def initialize(*args)
    super
    @log.level = Logger::Severity::DEBUG
    namespace = 'http://pragbouquet.com/stockPort'
    add_servant(Stock.new, namespace)
    add_request_headerhandler(ServerAuthHeaderHandler) 
  end
end


if $0 == __FILE__
  svr = StockServer.new('stock', nil, '0.0.0.0', 7000)
  trap(:INT) do
    svr.shutdown
  end
  status = svr.start
end
