#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'rubygems'
require 'active_record'

ActiveRecord::Base.establish_connection(
  :adapter => 'mysql',
  :host => '127.0.0.1',
  :database => 'webshop'
)

class Flower < ActiveRecord::Base
  belongs_to :stock_item
end

class StockItem < ActiveRecord::Base
  has_one :flower
end



class Stock
  def create_flower(name, price, quantity)
    flower = Flower.find_by_name(name)
    raise "#{name} already exists!" if !flower.nil?
    si = StockItem.new(:quantity => quantity)
    si.save
    si.create_flower(:name => name, :price => price)
    quantity
  end

  def add_flowers(name, quantity)
    adjust_quantity(name, quantity)
  end

  def remove_flowers(name, quantity)
    adjust_quantity(name, -quantity)
  end

  def set_price(name, price)
    flower = Flower.find_by_name(name)
    raise "#{name} is unknown!" if flower.nil?
    flower.price = price
    flower.save
    price
  end
  
  def get_report
    StockItem.find(:all).inject({}) do |r,si|
      r[si.flower.name] = [si.quantity, si.flower.price]; r
    end
  end
  
  private
  
  def adjust_quantity(name, difference)
    flower = Flower.find_by_name(name)
    raise "#{name} is unknown!" if flower.nil?
    si = StockItem.find(flower.stock_item_id)
    si.quantity += difference
    raise 'Not enough flowers!' if si.quantity < 0
    si.save
    si.quantity
  end
end

