#!/usr/bin/ruby -w
#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---


require 'socket'

# This class acts as a proxy to the PragBouquet Status Monitor.
class StatusMonitorClient
  WARN = 'w'
  ERROR = 'e'
  FATAL = 'f'

  # Create a new proxy for the Status Monitor.
  # host:: The host the Status Monitor is running on.
  # port:: The port the Status Monitor is listening on.   
  def initialize(host, port)
    @sm = TCPSocket.new(host, port)
  end
   
  def warn(app, msg) log(WARN, app, msg); end
  def error(app, msg) log(ERROR, app, msg); end
  def fatal(app, msg) log(FATAL, app, msg); end
   
  def terminate() @sm.close; end
   
  private
   
  def log(level, app, msg)
    @sm.puts [level, app, msg].join(',')
    @sm.readline
  end
end


if $0 == __FILE__
  
  sm = StatusMonitorClient.new('127.0.0.1', 3333)
  # ...
  sm.fatal('billing', 'Lost connection to payment gateway.')
  # ...
  sm.terminate
  
end
