#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---
SERVICE_NAME = 'StockObserver'
SERVICE_DISPLAYNAME = 'PragBouqet Observer'

$:.unshift File.dirname(__FILE__)

require 'win32/service'
include Win32

def install_service(name, display_name, executable)
  service = Service.new
  service.create_service do |s|
    s.service_name = name
    s.display_name = display_name
    s.binary_path_name = 'ruby ' + File.expand_path(executable) 
    s.dependencies = []
  end
  service.close
  puts "#{name} has been installed."
end



def start_service(name)
  Service.start(name)
  started = false
  while !started
    s = Service.status(name)
    started = (s.current_state == 'running')
    break if started
    puts 'Trying to start service...'
    sleep 1
  end
  puts "#{name} was started."
end



def stop_service(name)
  Service.stop(name)
  stopped = false
  while !stopped
    s = Service.status(name)
    stopped = (s.current_state == 'stopped')
    break if stopped
    puts 'Trying to stop service.'
    sleep 1
  end
  puts "#{name} was stopped."
end



def uninstall_service(name)
  begin
    Service.stop(name)
  rescue
  end
  Service.delete(name)
  puts "#{name} was uninstalled."
end



if __FILE__ == $0
  case ARGV[0]
  when 'install'
    install_service(SERVICE_NAME, SERVICE_DISPLAYNAME, ARGV[1])
  when 'start'
    start_service(SERVICE_NAME)
  when 'stop'
    stop_service(SERVICE_NAME)
  when 'uninstall'
    uninstall_service(SERVICE_NAME)
  else
    puts "Unknown command #{ARGV[0]}."
  end
end


