#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

$:.unshift File.dirname(__FILE__)
require 'status_monitor_client'
require 'logger'
require 'soap/rpc/driver'
require 'win32/service'
include Win32

class StockServerObserver < Daemon
  def initialize(opts = {})
    super()
    @opts = opts
  end
  
  def service_init 
    @interval = @opts[:interval] || 60
    @logger = Logger.new(@opts[:logfile] || 'c:/observer.log')
    @stock = SOAP::RPC::Driver.new(
      @opts[:soap_url] || 'http://localhost:2000',
      @opts[:soap_urn] || 'urn:Stock'
    )
    @stock.add_method('get_report')
    @sm = StatusMonitorClient.new(
      @opts[:sm_host] || '127.0.0.1',
      @opts[:sm_port] || 3333
    )
    @logger.info('Observer has been initialized.')
  end

  def test_stock_service
    @stock.get_report.class == Hash
  end

  def service_main
    @logger.info('Observer has been started.')
    sleep 1 while state != RUNNING 
    while state == RUNNING
      if !test_stock_service
        msg = 'Stock service is not running.'
        @logger.warn(msg)
        @sm.warn('stock', msg)
      else
        @logger.info('Stock service is running.')
      end
      sleep(@interval)
    end
    @logger.info('Observer has been stopped.')
  end
end


if __FILE__ == $0
  observer = StockServerObserver.new(:interval => 3)
  observer.mainloop
end

# vim:sw=2:tw=0

