#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'yaml'
require 'logger'
require 'fileutils'

class TestTool
  attr_accessor :create_reference
  attr_accessor :reference_dir, :output_dir

  def initialize(options)
    @create_reference = options[:create_reference]
    @reference_dir = options[:reference_dir]
    FileUtils.mkdir_p(@reference_dir) if @create_reference
    @output_dir = options[:output_dir]
    FileUtils.mkdir_p(@output_dir)
    @logger = Logger.new(options[:logfile])
  end

  def process(input_file)
    @logger.info("Processing #{input_file}.")
    test_suite = YAML::load_file(input_file)
    test_suite.each do |test_case|
      command = test_case['cmd']
      output_file = File.join(@output_dir, test_case['out'])
      process_test_case(command, output_file)
    end
  end
  def process_test_case(command, output_file)
    begin
      output = exec_command(command)
      @logger.info("Storing output in #{output_file}.")
      File.open(output_file, 'w') { |f| f.write(output) }
      reference_file = File.join(@reference_dir, output_file)
      if @create_reference
        FileUtils.mkdir_p(File.dirname(reference_file))
        FileUtils.cp(output_file, reference_file)
      else
        if !FileUtils.compare_file(output_file, reference_file)
          @logger.warn("#{output_file} differs from reference file!")
        else
          @logger.info("#{output_file} is correct.")
        end
      end
    rescue Exception => ex
      puts ex.backtrace
      @logger.error(ex.to_s)
    end
  end
  def exec_command(command)
    @logger.info("Running #{command}.")
    output = `#{command}`
    raise "An error occurred: #{$?}!" if $? != 0
    output
  end
end


if __FILE__ == $0

  tt = TestTool.new(
    :reference_dir => './reference',
    :create_reference => true,
    :output_dir => './output',
    :logfile => 'test_tool.log'
  )
  tt.process('suite1.yaml')


sleep 2


  tt.create_reference = false
  tt.process('suite1.yaml')

end
