#!/usr/local/bin/ruby -w
#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

require 'rexml/document'


def tag(tag_name, value, attrs ={})
  tmp = "<#{tag_name}"
  if attrs
    attrs.each { |k,v| tmp += " #{k}='#{v}'" }
  end
  tmp + ">#{value}</#{tag_name}>\n"
end

puts tag('hello', 'world')
puts tag('a', 'b', { 'c' => 'd' })



Address = Struct.new(:street, :city, :postal_code)

class Address
  def to_xml
    tag('address',
      tag('street', self.street) +
      tag('city', self.city) +
      tag('postal-code', self.postal_code)
    )
  end
end



Person = Struct.new(:name, :surname, :address)

class Person
  def to_xml
    tag('person', 
      self.address.to_xml, {
        'name' => self.name,
        'surname' => self.surname
      }
    )
  end
end



address = Address.new(
   'Musterstr. 42',
   'Berlin',
   '11011'
)
max_m = Person.new('Max', 'Mustermann', address)
puts max_m.to_xml



class Address

  undef_method :to_xml

  def to_xml
    adr = REXML::Element.new('address')
    adr.add_element('street').add_text(self.street)
    adr.add_element('city').add_text(self.city)
    adr.add_element('postal-code').add_text(self.postal_code)
    adr
  end
end

address.to_xml.write($stdout, 0)


puts


adr = address.to_xml
pos = REXML::Element.new('position')
pos.add_attribute('longitude', '12.345')
pos.add_attribute('latitude', '56.789')
adr.add_element(pos)
adr.write($stdout, 0)


puts


require 'xmlsimple'

class Address

  undef_method :to_xml

  def to_xml
    adr_hash = {}
    self.each_pair { |k,v| adr_hash[k.to_s] = [v] }
    XmlSimple.xml_out(adr_hash, { 'rootname' => 'address' })
  end
end

puts address.to_xml


# vim:sw=2:tw=0
