#!/usr/local/bin/ruby -w
#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---


require 'time'
require 'rexml/document'
include REXML

Event = Struct.new(:timestamp, :state, :consignee)

class Event
  def to_s 
    self.timestamp.strftime('%Y-%m-%d %H:%M') + ": " + self.state
  end
end

class ParcelHistory 
  attr_reader :tracking_no, :events
  def initialize(tracking_no)
    @tracking_no = tracking_no
    @events = []
  end
  def add_event(event) @events << event; end
end

class ParcelHistoryParser
  def initialize(source)
    @doc = Document.new(source)
  end
  def each_parcel
    @doc.root.each_element('parcel-history') do |ph|
      history = ParcelHistory.new(ph.attributes['tracking-no'])
      ph.each_element("event") do |event_element|
        history.add_event(to_event(event_element))
      end
      yield history 
    end
  end

  private

  def to_event(element)
    timestamp = Time::xmlschema(element.attributes['timestamp'])
    state = element.attributes['state']
    consignee = nil
    if !element.elements['consignee'].nil?
      element.elements['consignee'].text 
    end
    Event.new(timestamp, state, consignee)
  end
end

parser = ParcelHistoryParser.new(File.new('packages.xml'))
parser.each_parcel do |history|
  puts history.tracking_no + ":"
  history.events.each { |e| puts "  " + e.to_s }
end


# vim:sw=2:tw=0
