#!/usr/local/bin/ruby -w
#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---


require 'time'
require 'rexml/parsers/sax2parser'
require 'rexml/sax2listener'



Event = Struct.new(:timestamp, :state, :consignee)

class Event
  def to_s
    self.timestamp.strftime('%Y-%m-%d %H:%M') + ": " + self.state
  end
end

class ParcelHistory
  attr_reader :tracking_no, :events
  
  def initialize(tracking_no)
    @tracking_no = tracking_no
    @events = []
  end

  def add_event(event) @events << event; end
end


class ParcelHistoryListener
  include REXML::SAX2Listener

  def initialize
    @in_consignee = false
  end
  
  def start_element(uri, localname, tag_name, attrs)
    if tag_name == 'parcel-history'
      @parcel_history = ParcelHistory.new(attrs['tracking-no'])
    elsif tag_name == 'event'
      timestamp = Time::xmlschema(attrs['timestamp'])
      state = attrs['state']
      @event = Event.new(timestamp, state, nil)
    elsif tag_name == 'consignee'
      @in_consignee = true
    end
  end

  def end_element(uri, localname, tag_name)
    if tag_name == 'parcel-history'
      puts @parcel_history.tracking_no + ":"
      @parcel_history.events.each { |e| puts '  ' + e.to_s }
    elsif tag_name == 'event'
      @parcel_history.add_event(@event)
    elsif tag_name == 'consignee'
      @in_consignee = false
    end
  end



  def characters(value)
    if @in_consignee
      @event.consignee = '' unless @event.consignee
      @event.consignee << value
    end
  end
end

parser = REXML::Parsers::SAX2Parser.new(File.new('packages.xml'))
parser.listen(ParcelHistoryListener.new)
parser.parse


# vim:sw=2:tw=0
