#!/usr/local/bin/ruby -w
#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---

Event = Struct.new('Event', :timestamp, :state)
Order = Struct.new('Order', :tracking_no)

class TrackingService
end

class TrackingServiceProxy
  def initialize(host, port)
    @host, @port = host, port
  end

  def track(tracking_no)
    IO.read(tracking_no + '.xml')
  end
end

class TrackingParser
  def TrackingParser.parse(tracking_result)
    tracking_result
  end
end

class TrackingTool
  def initialize(db_connection, host, port)
    @tracking_service = TrackingServiceProxy.new(host, port)
    @db = db_connection
  end

  def run(orders)
    orders.each do |order|
      tracking_result = @tracking_service.track(order.tracking_no)
      state = TrackingParser.parse(tracking_result)
      puts state
    end
  end
end

if $0 == __FILE__
  tt = TrackingTool.new(nil, 'localhost', 80)
  ['package1', 'package2'].each do |tracking_no|
    tt.run([Order.new('package1'), Order.new('package2')])
  end
end

# vim:sw=2:tw=0
