#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---
require 'rexml/document'
include REXML


doc = Document.new(File.new('packages.xml'))


puts "--- Get second parcel-history."

puts XPath.match(doc, '/tracking-result/parcel-history')[1]


puts "--- Get second parcel-history with XPath index."

puts XPath.first(doc, '/tracking-result/parcel-history[2]')


puts "--- Get all consignees."

puts XPath.match(doc, '//consignee')


puts "--- Get all consignees."

puts XPath.match(doc, 'descendant-or-self::consignee')


puts "--- Get all delivered packages."

puts XPath.match(doc, '/tracking-result/parcel-history/event[@state="delivered"]')


puts "--- Get all delivered packages."

puts XPath.match(doc, '/tracking-result/parcel-history/event[attribute::state="delivered"]')


puts "--- Get all events on 2005-05-04."

puts XPath.match(doc, '//event[starts-with(@timestamp, "2005-05-04")]')


puts "--- Get all 'delivered' events on 2005-05-02."

puts XPath.match(
  doc,
  '//event[starts-with(@timestamp, "2005-05-02")][@state="delivered"]'
)


puts "--- Get all events on 2005-05-04."

puts XPath.match(doc, '//event[substring-before(@timestamp, "T")="2005-05-04")]')


puts "--- Get all nodes belonging to 2X43."

puts XPath.match(doc, '//parcel-history[@tracking-no="2X43"]/*')


puts "--- Get all tracking numbers."

puts XPath.match(doc, '//parcel-history/@tracking-no')


puts "--- Get all tracking numbers with axis."

puts XPath.match(doc, '//parcel-history/attribute::tracking-no')


puts "--- Get all different states."

states = XPath.match(doc, '//event/@state')
states.map! { |s| s.value }.uniq!
puts states


puts "--- Get all states and tracking numbers."

puts XPath.match(doc, '//event/@state | //parcel-history/@tracking-no')


puts "--- Get all delivered events."

puts XPath.match(
  doc,
  '//event[@state="delivered"]|//event[@state="picked-up"'
)


puts "--- Get first consignee."

puts XPath.match(doc, '/tracking-result/parcel-history/event/consignee[1]')


puts "--- Get all consignees (delivered) relatively."

events = XPath.match(
  doc,
  '/tracking-result/parcel-history/event[@state="delivered"]'
)
events.each do |event|
   puts XPath.match(event, 'consignee')
end


puts "--- Get first consignee's XPath."

event = XPath.first(
  doc,
  '/tracking-result/parcel-history/event[@state="delivered"]'
)
puts event.xpath

__END__

# Those that doesn't work
puts "--- Get number of delivered packages."
puts XPath.match(doc, 'count(/tracking-result/parcel-history/event[@state="delivered"])')

puts "--- Do some math."
puts XPath.match(doc, '42 * 5')
